from dataclasses import dataclass
from typing import Any, List, Union

__all__ = [
    'RulesList',
    'Rule',
    'Option',
    'Variable',
    'Setting',
    'NegatedSetting',
    'Negated',
    'PortRange',
    'Grouping',
]


class RulesList(List['Rule']):
    def __str__(self):
        return '\n\n'.join(str(rule) for rule in self)


@dataclass
class Rule:
    action: str
    protocol: str
    src: Any
    src_port: Any
    direction: str
    dst: Any
    dst_port: Any
    options: List['Option']

    def __str__(self):
        return '''
            {action} {protocol} {src} {src_port} {direction} {dst} {dst_port} ( \\\n  {body} \\\n)
        '''.format(
            action=self.action,
            protocol=self.protocol,
            src=self.src,
            src_port=self.src_port,
            direction=self.direction,
            dst=self.dst,
            dst_port=self.dst_port,
            body=' \\\n  '.join(str(option) for option in self.options),
        ).strip()


@dataclass
class Option:
    keyword: str
    settings: 'Setting' = None

    def __str__(self):
        if self.settings is None:
            return f'{self.keyword};'
        else:
            return f'{self.keyword}: {self.settings};'


@dataclass(frozen=True)
class Variable:
    identifier: str

    def __str__(self):
        return f'${self.identifier}'


class Setting(str):
    @property
    def is_negated(self):
        return False


class NegatedSetting(Setting):
    @property
    def is_negated(self):
        return True

    def __repr__(self):
        return f'!{super().__repr__()}'


@dataclass(frozen=True)
class Negated:
    value: Any

    def __str__(self):
        return f'!{self.value}'

    def __eq__(self, other):
        return self.value != other

    def __contains__(self, item):
        return item not in self.value


@dataclass
class PortRange:
    start: Union[Variable, int] = None
    end: Union[Variable, int] = None

    def __str__(self):
        return ''.join(
            str(component)
            for component in (self.start, ':', self.end)
            if component is not None
        )

    def __contains__(self, item):
        if self.start is not None and item < self.start:
            return False

        if self.end is not None and item > self.end:
            return False

        return True


class Grouping(list):
    def __str__(self):
        return f'[{", ".join(str(item) for item in self)}]'



__version__ = "0.2.2"

from ._parser import parser
from .rules import *
from .transformer import RuleTransformer


def parse_rules(source: str) -> RulesList:
    tree = parser.parse(source)
    return RuleTransformer().transform(tree)



from ipaddress import ip_address, ip_interface
from typing import Any, Union

from lark import InlineTransformer, Token, Tree

from .rules import (
    Grouping,
    Negated,
    NegatedSetting,
    Option,
    PortRange,
    Rule,
    RulesList,
    Setting,
    Variable,
)


class RuleTransformer(InlineTransformer):
    str = str

    def rules(self, *rules):
        return RulesList(rules)

    def rule(self,
             action,
             protocol,
             src,
             src_port,
             direction,
             dst,
             dst_port,
             body
             ) -> Rule:
        return Rule(
            action,
            protocol,
            src,
            src_port,
            direction,
            dst,
            dst_port,
            body.children,
        )

    action = str
    protocol = str
    any = str
    direction = str

    def port_grouping(self, *components):
        return Grouping(components)

    def port_range(self, *components):
        if len(components) == 3:
            start, colon, end = components
        elif isinstance(components[0], int):
            (start, colon), end = components, None
        else:
            start, (colon, end) = None, components
        return PortRange(start, end)

    def option(self, keyword: Token, settings: Union[Token, Tree] = None) -> Option:
        if settings is not None:
            value = settings
        else:
            value = None

        return Option(str(keyword), value)

    def settings(self, value: Any):
        return Setting(value)

    def negated_settings(self, value: Any):
        return NegatedSetting(value)

    def variable(self, variable: Token):
        identifier = str(variable)[1:]
        return Variable(identifier)

    def ip(self, tok: Token):
        return ip_address(str(tok))

    def cidr(self, tok: Token):
        return ip_interface(str(tok))

    def string(self, tok: Token):
        return tok.value[1:-1]

    def integer(self, tok):
        return int(tok.value)

    def negated(self, value: Any):
        return Negated(value)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['parsuricata']

package_data = \
{'': ['*']}

install_requires = \
['lark-parser>=0.7.1,<0.8.0']

setup_kwargs = {
    'name': 'parsuricata',
    'version': '0.2.2',
    'description': 'Parse Suricata rules',
    'long_description': '# parsuricata\n\nParse Suricata rules\n\n\n# Installation\n\n```bash\npip install parsuricata\n```\n\n\n# Usage\n\n```python\nfrom parsuricata import parse_rules\n\nsource = \'\'\'\n  alert http $HOME_NET any -> !$HOME_NET any (msg: "hi mum!"; content: "heymum"; http_uri; sid: 1;)\n\'\'\'\n\nrules = parse_rules(source)\nprint(rules)\n#\n# alert http $HOME_NET any -> !$HOME_NET any ( \\\n#   msg: hi mum!; \\\n#   content: heymum; \\\n#   http_uri; \\\n#   sid: 1; \\\n# )\n\nrule = rules[0]\n\nprint(rule.action)\n# alert\n\nprint(rule.protocol)\n# http\n\nprint(rule.src)\n# $HOME_NET\n\nprint(rule.src_port)\n# any\n\nprint(rule.direction)\n# ->\n\nprint(rule.dst)\n# !$HOME_NET\n\nprint(rule.dst_port)\n# any\n\nfor option in rule.options:\n    print(f\'{option.keyword} = {option.settings}\')\n#\n# msg = hi mum!\n# content = heymum\n# http_uri = None\n# sid = 1\n```\n',
    'author': 'Zach "theY4Kman" Kanzler',
    'author_email': 'they4kman@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
